'use strict';Object.defineProperty(exports,'__esModule',{value:true});var throttled = function throttled(handler, timeout) {
  if (!timeout) return handler;
  var active = false;
  var event = null;
  return function (e) {
    event = e;
    if (active) return;
    active = true;
    setTimeout(function () {
      handler(event);
      active = false;
    }, timeout);
  };
};

var event = function event(_event) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      target = _ref.target,
      throttle = _ref.throttle;

  var funcName = "_".concat(_event, "Handler");
  var deactivatedName = "_".concat(_event, "Deactivated");
  if (!target) target = window;
  return {
    mounted: function mounted() {
      this[funcName] = throttled(this[_event + 'Handler'], throttle);
      target.addEventListener(_event, this[funcName]);
    },
    destroyed: function destroyed() {
      target.removeEventListener(_event, this[funcName]);
    },
    activated: function activated() {
      if (!this[deactivatedName]) return;
      target.addEventListener(_event, this[funcName]);
      this[deactivatedName] = false;
    },
    deactivated: function deactivated() {
      target.removeEventListener(_event, this[funcName]);
      this[deactivatedName] = true;
    }
  };
};
var scroll = event('scroll', {
  throttle: 17
});
var resize = event('resize', {
  throttle: 17
});var eps = 0.05;

var isNumber = function isNumber() {
  for (var _len = arguments.length, values = new Array(_len), _key = 0; _key < _len; _key++) {
    values[_key] = arguments[_key];
  }

  return values.every(function (value) {
    return typeof value == 'number';
  });
};

var pxEq = function pxEq(first, second) {
  return isNumber(first, second) && Math.abs(first - second) < eps;
};
var pxGt = function pxGt(first, second) {
  return isNumber(first, second) && first - second > eps;
};//
var script = {
  mixins: [scroll, resize],
  props: {
    // Array of data that is passed to item
    itemsData: {
      type: Array,
      required: true
    },
    // Approximate item height (used at first render)
    itemHeight: {
      type: Number,
      required: true
    },
    // Vue js item component
    itemComponent: {
      type: [Object, Function],
      required: true
    },
    // Height of rendered part relative to viewport height
    renderViewports: {
      type: Number,
      default: 3
    },
    // The name of the item component function that is called on item resize
    onItemResize: String
  },
  data: function data() {
    return {
      start: 0,
      // index of the first rendered item
      end: 0,
      // index of the last rendered item (not included)
      height: 0,
      // average item height
      spacerMargin: 0 // margin before spacer element

    };
  },
  computed: {
    visibleItems: function visibleItems() {
      var _this = this;

      return this.itemsData.slice(this.start, this.end).map(function (v, i) {
        return {
          data: v,
          index: _this.start + i
        };
      });
    },
    spacerStyle: function spacerStyle() {
      return {
        transform: 'translateY(' + this.spacerMargin + 'px)'
      };
    },
    listStyle: function listStyle() {
      return {
        height: this.height + 'px',
        position: 'relative',
        overflow: 'hidden',
        overflowAnchor: 'none' // onItemsResize does this

      };
    }
  },
  methods: {
    /**
     * Scroll event handler
     * Checks scrolled distance. If the distance is within rendered items,
     * it will change start and spacerMargin according to known (or average)
     * items sizes.
     * If the scrolled distance is bigger, it will change start
     * and spacerMargin to approximate values using average item height
     */
    scrollHandler: function scrollHandler() {
      var _this2 = this;

      var prevScrollTop = this.scrollTop;
      this.scrollTop = Math.max(0, -this.$refs.list.getBoundingClientRect().top);
      var diff = this.scrollTop - prevScrollTop;
      var preRendered = Math.ceil(Math.ceil(window.innerHeight / this.average) * (this.renderViewports - 1) / 2);

      var height = function height(index) {
        return _this2.heights.get(index) || _this2.average;
      };

      var heights = function heights(index) {
        var down = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
        var res = 0;
        var max = index + preRendered + (down ? 1 : 0);

        for (var i = index; i < max; i++) {
          res += height(i);
        }

        return res;
      };

      if (Math.abs(diff) < 0.65 * window.innerHeight * this.renderViewports) {
        var spacerMargin = this.spacerMargin,
            start = this.start;

        if (diff > 0) {
          while (spacerMargin + heights(start, true) < this.scrollTop) {
            spacerMargin += height(start);
            start++;
          }
        } else {
          while (start > 0 && spacerMargin + heights(start) > this.scrollTop) {
            start--;
            spacerMargin -= height(start);
          }
        }

        this.spacerMargin = spacerMargin;
        this.setStart(start);
      } else {
        var _start = Math.floor(this.scrollTop / this.average);

        this.spacerMargin = _start * this.average;
        this.setStart(_start);
      }
    },

    /**
     * Resize event handler
     * Clears known items' heights if window width changes
     * Keeps visible heights
     */
    resizeHandler: function resizeHandler() {
      if (!pxEq(this.prevWidth, this.width())) {
        var old = this.heights;
        this.heights = new Map();

        for (var i = this.start; i < this.end; i++) {
          if (old.has(i)) this.heights.set(i, old.get(i));
        }
      }
    },

    /**
     * Getting real item height from dom
     * Uses the distance between bottoms of current and previous items.
     * If it's the first rendered item, uses top of the list element
     * It allows to use items with margins
     */
    getItemHeight: function getItemHeight(index) {
      var spacer = this.$refs.spacer;
      index -= this.start;
      var prev = index === 0 ? this.$refs.list.getBoundingClientRect().top + this.spacerMargin : spacer.children[index - 1].getBoundingClientRect().bottom;
      return spacer.children[index].getBoundingClientRect().bottom - prev;
    },

    /**
     * Returns true if item's top is invisible
     * Used to check if it needs to correct spacerMargin when actual size
     * of items changes.
     */
    isTopItem: function isTopItem(index) {
      var spacer = this.$refs.spacer;
      return spacer.children[index - this.start].getBoundingClientRect().top > 0;
    },

    /**
     * ResizeObserver handler
     * Updates average items' height and corrects spacerMargin
     */
    onItemsResize: function onItemsResize(entities) {
      var _this3 = this;

      if (this.isDeactivated) return;
      var average = this.average;
      average *= this.heights.size;
      var changed = false;
      var spacerMargin = 0;
      entities.forEach(function (entity) {
        var el = entity.target;
        var idx = Number(el.dataset.itemIndex);

        if (idx >= _this3.start && idx < _this3.end) {
          var prevHeight = _this3.heights.get(idx);

          var newHeight = _this3.getItemHeight(idx);

          if (!pxEq(prevHeight, newHeight)) {
            if (!_this3.isTopItem(idx)) spacerMargin += newHeight - (prevHeight || _this3.average);
            average += newHeight - (prevHeight || 0);

            _this3.heights.set(idx, newHeight);

            changed = true;
          }

          if (_this3.onItemResize) el.__vue__[_this3.onItemResize]();
        }
      });

      if (changed) {
        this.average = average / this.heights.size;
        this.spacerMargin -= spacerMargin;
        this.setStart(this.start);
      }

      this.$nextTick(function () {
        _this3.fixSpacerMargin();

        _this3.fixHeight();
      });
    },
    observe: function observe(from, to) {
      var _this4 = this;

      this.$nextTick(function () {
        for (var i = from; i < to; i++) {
          _this4.observer.observe(_this4.$refs.spacer.children[i]);
        }
      });
    },
    unobserve: function unobserve(from, to) {
      for (var i = from; i < to; i++) {
        this.observer.unobserve(this.$refs.spacer.children[i]);
      }
    },

    /**
     * Sets a new start index
     * Calculates end index and updates observed items in ResizeObserver
     */
    setStart: function setStart(index) {
      this.prevWidth = this.width();
      var count = Math.round(this.renderViewports * Math.ceil(window.innerHeight / this.average));
      var prevStart = this.start,
          prevEnd = this.end;
      this.start = Math.max(index, 0);
      this.end = Math.min(this.start + count, this.itemsData.length);
      if (prevStart === this.start && prevEnd === this.end) return;
      if (this.start != prevStart || this.end != prevEnd) this.unobserve(0, Math.min(this.start, prevEnd) - prevStart);
      if (prevEnd > this.end) this.unobserve(Math.max(this.end, prevStart) - prevStart, prevEnd - prevStart);
      if (this.start < prevStart) this.observe(0, Math.min(prevStart, this.end) - this.start);
      if (prevEnd < this.end) this.observe(Math.max(prevEnd, this.start) - this.start, this.end - this.start);
      var bottom = this.itemsData.length === this.end;
      if (bottom && !this.bottom) this.$emit('bottom');
      this.bottom = bottom;
    },

    /**
     * Fixes spacerMargin errors
     */
    fixSpacerMargin: function fixSpacerMargin() {
      if (this.start === 0) {
        if (pxEq(this.spacerMargin, 0)) return;
        window.scroll(0, window.scrollY - this.spacerMargin);
        this.spacerMargin = 0;
      } else if (pxGt(0, this.spacerMargin)) {
        var avMargin = this.spacerMargin - this.average * this.start;
        this.spacerMargin -= avMargin;
        window.scroll(0, window.scrollY - avMargin);
      }
    },

    /**
     * Fixes height error
     */
    fixHeight: function fixHeight() {
      var _this$$refs = this.$refs,
          list = _this$$refs.list,
          spacer = _this$$refs.spacer;
      var diff = spacer.getBoundingClientRect().bottom - list.getBoundingClientRect().bottom;

      if (this.itemsData.length == this.end) {
        if (pxEq(diff, 0)) return;
        this.height += diff;
      } else if (pxGt(diff, 0)) {
        this.height = Math.max(this.height + diff, this.average * this.itemsData.length);
      } else if (Math.abs(this.height - this.average * this.itemsData.length) > 0.1 * this.itemsData.length * this.average) {
        this.height = this.average * this.itemsData.length;
      }
    },
    width: function width() {
      var list = this.$refs.list;
      return list ? list.getBoundingClientRect().width : 0;
    }
  },
  watch: {
    itemsData: function itemsData() {
      this.setStart(this.start);
    }
  },
  created: function created() {
    this.heights = new Map();
    this.scrollTop = 0;
    this.average = this.itemHeight;
    this.observer = new ResizeObserver(this.onItemsResize);
    this.setStart(0);
  },
  beforeDestroy: function beforeDestroy() {
    this.observer.disconnect();
  },
  activated: function activated() {
    this.isDeactivated = false;
  },
  deactivated: function deactivated() {
    this.isDeactivated = true;
  }
};function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}/* script */
var __vue_script__ = script;
/* template */

var __vue_render__ = function __vue_render__() {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', {
    ref: "list",
    style: _vm.listStyle
  }, [_vm._ssrNode("<div" + _vm._ssrStyle(null, _vm.spacerStyle, null) + ">", "</div>", _vm._l(_vm.visibleItems, function (item) {
    return _c(_vm.itemComponent, {
      key: item.index,
      tag: "component",
      attrs: {
        "data-item-index": item.index,
        "data": item.data,
        "index": item.index
      }
    });
  }), 1)]);
};

var __vue_staticRenderFns__ = [];
/* style */

var __vue_inject_styles__ = undefined;
/* scoped */

var __vue_scope_id__ = undefined;
/* module identifier */

var __vue_module_identifier__ = "data-v-9ff47b10";
/* functional template */

var __vue_is_functional_template__ = false;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

var __vue_component__ = /*#__PURE__*/normalizeComponent({
  render: __vue_render__,
  staticRenderFns: __vue_staticRenderFns__
}, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__, __vue_module_identifier__, false, undefined, undefined, undefined);// Import vue component

var install = function installListScroller(Vue) {
  if (install.installed) return;
  install.installed = true;
  Vue.component('ListScroller', __vue_component__);
}; // Create module definition for Vue.use()


var plugin = {
  install: install
}; // To auto-install on non-es builds, when vue is found
// eslint-disable-next-line no-redeclare

/* global window, global */

{
  var GlobalVue = null;

  if (typeof window !== 'undefined') {
    GlobalVue = window.Vue;
  } else if (typeof global !== 'undefined') {
    GlobalVue = global.Vue;
  }

  if (GlobalVue) {
    GlobalVue.use(plugin);
  }
} // Inject install function into component - allows component
// to be registered via Vue.use() as well as Vue.component()


__vue_component__.install = install; // Export component by default
// also be used as directives, etc. - eg. import { RollupDemoDirective } from 'rollup-demo';
// export const RollupDemoDirective = component;
exports.default=__vue_component__;